# Akismet.dart
Prevent comment spam using [Akismet](https://akismet.com) service, in [Dart](https://www.dartlang.org).

## Features
- [Key verification](https://akismet.com/development/api/#verify-key): checks an Akismet API key and gets a value indicating whether it is valid.
- [Comment check](https://akismet.com/development/api/#comment-check): checks a comment and gets a value indicating whether it is spam.
- [Submit spam](https://akismet.com/development/api/#submit-spam): submits a comment that was not marked as spam but should have been.
- [Submit ham](https://akismet.com/development/api/#submit-ham): submits a comment that was incorrectly marked as spam but should not have been.

## Documentation
- [API Reference](http://dev.belin.io/akismet.dart/api)

## Installing via [Pub](https://pub.dartlang.org)

#### 1. Depend on it
Add this to your package's `pubspec.yaml` file:

```yaml
dependencies:
  akismet: any
```

#### 2. Install it
If you're using the [Dart Editor](https://www.dartlang.org/tools/editor), choose:

```
Menu > Tools > Pub Get
```

Or if you want to install from the command line, run:

```shell
$ pub get
```

#### 3. Import it
Now in your Dart code, you can use:

```dart
import 'package:akismet/html.dart'; // In browser applications.
import 'package:akismet/io.dart'; // In console applications.
```

## Usage

#### Key Verification

```dart
var client = new Client('123YourAPIKey', Uri.parse('http://your.blog.url'));
client.verifyKey().then((isValid) =>
  print(isValid ? 'Your API key is valid.' : 'Your API key is invalid.')
);
```

#### Comment Check

```dart
var comment = new Comment('A comment.', new Author('An author.'));
client.checkComment(comment).then((isSpam) =>
  print(isSpam ? 'The comment is marked as spam.' : 'The comment is marked as ham.')
);
```

#### Submit Spam/Ham

```dart
client.submitSpam(comment).then((_) => print('Spam submitted.'));
client.submitHam(comment).then((_) => print('Ham submitted.'));
```

## Implementations

#### Client
The Akismet client comes in two flavors: a first one based on [`dart:io`](https://api.dartlang.org/apidocs/channels/stable/dartdoc-viewer/dart-io)
for server/console applications, and a second one based on [`dart:html`](https://api.dartlang.org/apidocs/channels/stable/dartdoc-viewer/dart-dom-html)
for client/browser applications.

Their usage is the same, but the HTML client is limited by security restrictions in a browser context.
Unfortunately, the [Akismet](https://akismet.com) service does not support [CORS](http://www.w3.org/TR/cors) headers.
So, the HTML client can't be used directly with the official service.

#### Server
To be able to use the HTML client, we must rely on a proxy server adding [CORS](http://www.w3.org/TR/cors) headers to service responses.

This is why a [server implementation](https://github.com/cedx/akismet.dart/blob/master/lib/src/io/server.dart) is provided with this package.
To facilitate its usage, a [command line interface](https://github.com/cedx/akismet.dart/blob/master/bin/cli.dart) is available in the `bin` folder.

From a command prompt, run the `cli.dart` script:

```
$ dart bin/cli.dart --help

Starts an Akismet.dart server.

Usage:
    cli.dart [options]

Options:
-a, --address     The address to which to listen.
                  (defaults to "0.0.0.0")

-p, --port        The port on which to listen.
                  (defaults to "3000")

-r, --redirect    The URL to redirect when a request is not handled.
-h, --help        Print this usage information.
```

## License
[Akismet.dart](https://pub.dartlang.org/packages/akismet) is distributed under the MIT License.
