#!/usr/bin/env dart

/**
 * Command line interface.
 * @module bin.cli
 */
library akismet.cli;

import 'dart:io';
import 'package:akismet/io.dart';
import 'package:args/args.dart';
import 'package:path/path.dart' as path;

/**
 * Represents an application providing functionalities specific to console requests.
 * @class cli.Application
 * @static
 */

/**
 * Object used to parse command line argments.
 * @property _parser
 * @type args.ArgParser
 * @static
 * @private
 */
final ArgParser _parser=new ArgParser()
  ..addOption('address', abbr: 'a', defaultsTo: '0.0.0.0', help: 'The address to which to listen.')
  ..addOption('port', abbr: 'p', defaultsTo: '3000', help: 'The port on which to listen.')
  ..addOption('redirect', abbr: 'r', help: 'The URL to redirect when a request is not handled.')
  ..addFlag('help', abbr: 'h', help: 'Print this usage information.', negatable: false);

/**
 * Application entry point.
 * @method main
 * @param {List&lt;String&gt;} arguments The command line arguments.
 * @static
 */
void main(List<String> arguments) {
  try {
    var results=_parser.parse(arguments);
    if(results['help']) return printUsage();

    var redirectUrl=(results['redirect']!=null ? Uri.parse(results['redirect']) : null);
    return startServer(results['address'], int.parse(results['port']), redirectUrl: redirectUrl);
  }

  on FormatException catch(e) {
    print(e.message);
    exit(1);
  }
}

/**
 * Prints the usage information.
 * @method printUsage
 * @static
 */
void printUsage() {
  var script=path.basename(Platform.script.toFilePath());
  var buffer=new StringBuffer()
    ..writeln('Starts an Akismet.dart server.')
    ..writeln()
    ..writeln('Usage:')
    ..writeln('    $script [options]')
    ..writeln()
    ..writeln('Options:')
    ..write(_parser.getUsage());

  print(buffer);
}

/**
 * Starts a server listening for HTTP requests.
 * @method startServer
 * @param {String|InternetAddress} address The address that the server is listening on.
 * @param {int} port The port that the server is listening on.
 * @param {Uri} [redirectUrl] The URL to redirect the user when a request is unhandled.
 * @static
 */
void startServer(address, int port, { Uri redirectUrl }) {
  var server=new Server(redirectUrl);
  server.start(address, port).then((_) {
    var now=new DateTime.now();
    print('[$now] Server started on http://${server.address.address}:${server.port}.');
  });
}
