/**
 * Implementation of the `akismet.core.Client` class.
 * @module core.client
 */
part of akismet.core;

/**
 * Provides the base class for clients that submit comments to [Akismet](https://akismet.com) service.
 * @class akismet.core.Client
 * @constructor
 * @param {String} apiKey The Akismet API key.
 * @param {Uri} blog The front page or home URL.
 */
abstract class Client {

  Client(this.apiKey, this.blog) {
    assert(apiKey!=null);
    assert(blog!=null);
  }

  /**
   * The Akismet API key.
   * @property apiKey
   * @type String
   * @final
   */
  final String apiKey;

  /**
   * The front page or home URL of the instance making requests.
   * @property blog
   * @type Uri
   * @final
   */
  final Uri blog;

  /**
   * The user agent string to use when making requests.
   * If possible, the user agent string should always have the following format: `Application Name/Version | Plugin Name/Version`.
   * @property userAgent
   * @type String
   */
  String userAgent;

  /**
   * Checks the specified comment against the service database.
   * @method checkComment
   * @param {akismet.core.Comment} comment The comment to be checked.
   * @return {Future&lt;bool&gt;} Value indicating whether it is spam.
   * @async
   */
  Future<bool> checkComment(Comment comment);

  /**
   * Submits the specified comment that was incorrectly marked as spam but should not have been.
   * @method submitHam
   * @param {akismet.core.Comment} comment The comment to be submitted.
   * @return {Future} A `Future` that completes once the comment has been submitted.
   * @async
   */
  Future submitHam(Comment comment);

  /**
   * Submits the specified comment that was not marked as spam but should have been.
   * @method submitSpam
   * @param {akismet.core.Comment} comment The comment to be submitted.
   * @return {Future} A `Future` that completes once the comment has been submitted.
   * @async
   */
  Future submitSpam(Comment comment);

  /**
   * Returns a string representation of this object.
   * @method toString
   * @return {String} A string that represents the current object.
   */
  @override
  String toString() => 'Client { apiKey: "$apiKey", blog: Uri($blog), userAgent: "$userAgent" }';

  /**
   * Checks the `apiKey` against the service database.
   * @method verifyKey
   * @return {Future&lt;bool&gt;} Value indicating whether it is a valid API key.
   * @async
   */
  Future<bool> verifyKey();
}
