/**
 * Implementation of the `akismet.html.Client` class.
 * @module html.client
 */
part of akismet.html;

/**
 * A `dart:html` client that submit comments to [Akismet](https://akismet.com) service.
 * @class akismet.html.Client
 * @extends akismet.core.Client
 * @constructor
 * @param {String} apiKey The Akismet API key.
 * @param {Uri} blog The front page or home URL.
 */
class Client extends core.Client {

  Client(String apiKey, Uri blog): super(apiKey, blog) {
    userAgent='Dart/0.0.0 | Akismet/${core.VERSION}';
  }

  /**
   * The URL of the remote service.
   * @property serviceUrl
   * @type Uri
   * @default "http://127.0.0.1:8080"
   */
  Uri serviceUrl=Uri.parse('http://127.0.0.1:8080');

  /**
   * Checks the specified comment against the service database.
   * @method checkComment
   * @param {akismet.core.Comment} comment The comment to be checked.
   * @return {Future&lt;bool&gt;} Value indicating whether it is spam.
   * @async
   */
  Future<bool> checkComment(core.Comment comment) {
    assert(comment!=null);
    var endPoint=serviceUrl.resolve(core.EndPoints.checkComment.pattern);
    return _queryService(endPoint, comment.toJson()).then((result) => result=='true');
  }

  /**
   * Submits the specified comment that was incorrectly marked as spam but should not have been.
   * @method submitHam
   * @param {akismet.core.Comment} comment The comment to be submitted.
   * @return {Future} A `Future` that completes once the comment has been submitted.
   * @async
   */
  Future submitHam(core.Comment comment) {
    assert(comment!=null);
    var endPoint=serviceUrl.resolve(core.EndPoints.submitHam.pattern);
    return _queryService(endPoint, comment.toJson());
  }

  /**
   * Submits the specified comment that was not marked as spam but should have been.
   * @method submitSpam
   * @param {akismet.core.Comment} comment The comment to be submitted.
   * @return {Future} A `Future` that completes once the comment has been submitted.
   * @async
   */
  Future submitSpam(core.Comment comment) {
    assert(comment!=null);
    var endPoint=serviceUrl.resolve(core.EndPoints.submitSpam.pattern);
    return _queryService(endPoint, comment.toJson());
  }

  /**
   * Checks the `apiKey` against the service database.
   * @method verifyKey
   * @return {Future&lt;bool&gt;} Value indicating whether it is a valid API key.
   * @async
   */
  Future<bool> verifyKey() {
    var endPoint=serviceUrl.resolve(core.EndPoints.verifyKey.pattern);
    return _queryService(endPoint, {}).then((result) => result=='valid');
  }

  /**
   * Queries the remote service by posting the specified fields to a given end point.
   * Throws a `akismet.html.HttpException` if the remote service returned an error message.
   * @method _queryService
   * @return {Future&lt;String&gt;} The response as a string.
   * @async
   * @private
   */
  Future<String> _queryService(Uri endPoint, Map<String, String> fields) {
    assert(endPoint!=null);
    assert(fields!=null);

    fields['blog']=blog.toString();
    fields['key']=apiKey;

    var headers={
      core.HttpHeaders.REQUESTED_WITH: 'XMLHttpRequest',
      core.HttpHeaders.USER_AGENT: userAgent
    };

    return HttpRequest.postFormData(endPoint.toString(), fields, requestHeaders: headers, responseType: 'text').then((request) {
      if(request.responseHeaders.containsKey(core.HttpHeaders.AKISMET_DEBUG_HELP))
        throw new HttpException(request.responseHeaders[core.HttpHeaders.AKISMET_DEBUG_HELP], uri: endPoint);

      return request.responseText;
    });
  }
}

/**
 * Describes an exception that occurred during the processing of HTTP requests.
 * @class akismet.html.HttpException
 * @uses Exception
 * @constructor
 * @param {String} message A message that describes the current exception.
 * @param {Uri} [uri] The requested `Uri` that caused the exception.
 */
class HttpException implements Exception {

  const HttpException(this.message, { this.uri });

  /**
   * A message that describes the current exception.
   * @property message
   * @type String
   * @final
   */
  final String message;

  /**
   * The requested `Uri` that caused the exception.
   * @property uri
   * @type Uri
   * @final
   */
  final Uri uri;

  /**
   * Returns a string representation of this object.
   * @method toString
   * @return {String} A string that represents the current object.
   */
  @override
  String toString() => 'HttpException { message: "$message", uri: Uri($uri) }';
}
