/**
 * Implementation of the `akismet.io.Client` class.
 * @module io.client
 */
part of akismet.io;

/**
 * A `dart:io` client that submit comments to [Akismet](https://akismet.com) service.
 * @class akismet.io.Client
 * @extends akismet.core.Client
 * @constructor
 * @param {String} apiKey The Akismet API key.
 * @param {Uri} blog The front page or home URL.
 */
class Client extends core.Client {

  Client(String apiKey, Uri blog): super(apiKey, blog) {
    var dartVersion=Platform.version.split('.').take(3).join('.');
    userAgent='Dart/$dartVersion | Akismet/${core.VERSION}';
  }

  /**
   * A value indicating whether to use secure requests when querying the service database.
   * @property useSecureRequests
   * @type bool
   * @default false
   */
  bool useSecureRequests=false;

  /**
   * Checks the specified comment against the service database.
   * @method checkComment
   * @param {akismet.core.Comment} comment The comment to be checked.
   * @return {Future&lt;bool&gt;} Value indicating whether it is spam.
   * @async
   */
  Future<bool> checkComment(core.Comment comment) {
    assert(comment!=null);
    var endPoint=new Uri(scheme: useSecureRequests ? 'https' : 'http', host: '$apiKey.rest.akismet.com', path: core.EndPoints.checkComment.pattern);
    return _queryService(endPoint, comment.toJson()).then((result) => result=='true');
  }

  /**
   * Submits the specified comment that was incorrectly marked as spam but should not have been.
   * @method submitHam
   * @param {akismet.core.Comment} comment The comment to be submitted.
   * @return {Future} A `Future` that completes once the comment has been submitted.
   * @async
   */
  Future submitHam(core.Comment comment) {
    assert(comment!=null);
    var endPoint=new Uri(scheme: useSecureRequests ? 'https' : 'http', host: '$apiKey.rest.akismet.com', path: core.EndPoints.submitHam.pattern);
    return _queryService(endPoint, comment.toJson());
  }

  /**
   * Submits the specified comment that was not marked as spam but should have been.
   * @method submitSpam
   * @param {akismet.core.Comment} comment The comment to be submitted.
   * @return {Future} A `Future` that completes once the comment has been submitted.
   * @async
   */
  Future submitSpam(core.Comment comment) {
    assert(comment!=null);
    var endPoint=new Uri(scheme: useSecureRequests ? 'https' : 'http', host: '$apiKey.rest.akismet.com', path: core.EndPoints.submitSpam.pattern);
    return _queryService(endPoint, comment.toJson());
  }

  /**
   * Checks the `apiKey` against the service database.
   * @method verifyKey
   * @return {Future&lt;bool&gt;} Value indicating whether it is a valid API key.
   * @async
   */
  Future<bool> verifyKey() {
    var endPoint=new Uri(scheme: useSecureRequests ? 'https' : 'http', host: 'rest.akismet.com', path: core.EndPoints.verifyKey.pattern);
    return _queryService(endPoint, { 'key': apiKey }).then((result) => result=='valid');
  }

  /**
   * Queries the remote service by posting the specified fields to a given end point.
   * Throws a `HttpException` if the remote service returned an error message.
   * @method _queryService
   * @return {Future&lt;String&gt;} The response as a string.
   * @async
   * @private
   */
  Future<String> _queryService(Uri endPoint, Map<String, String> fields) {
    assert(endPoint!=null);
    assert(fields!=null);

    fields['blog']=blog.toString();

    return http.post(endPoint, body: fields, headers: { HttpHeaders.USER_AGENT: userAgent }).then((response) {
      if(response.headers.containsKey(core.HttpHeaders.AKISMET_DEBUG_HELP))
        throw new HttpException(response.headers[core.HttpHeaders.AKISMET_DEBUG_HELP], uri: endPoint);

      return response.body;
    });
  }
}
