/**
 * Implementation of the `akismet.io.Server` class.
 * @module io.server
 */
part of akismet.io;

/**
 * An HTTP server that acts as a proxy between HTML clients and [Akismet](https://akismet.com) service.
 * @class akismet.io.Server
 * @constructor
 * @param {Uri} [redirectUrl] The URL to redirect the user when a request is unhandled.
 */
class Server {

  Server([ this.redirectUrl ]);

  /**
   * The underlying `HttpServer` instance.
   * @property _server
   * @type HttpServer
   * @private
   */
  HttpServer _server;

  /**
   * The address that the server is listening on, or `null` if the server is not started.
   * @property address
   * @type InternetAddress
   * @final
   */
  InternetAddress get address => _server!=null ? _server.address : null;

  /**
   * The port that the server is listening on, or `-1` if the server is not started.
   * @property port
   * @type int
   * @final
   */
  int get port => _server!=null ? _server.port : -1;

  /**
   * The URL to redirect the user when a request is unhandled.
   * If this property is `null`, a 404 status code is sent instead of redirecting.
   * @property redirectUrl
   * @type Uri
   * @final
   */
  final Uri redirectUrl;

  /**
   * Checks a `Comment` against the service database.
   * @method checkComment
   * @param {http_server.HttpRequestBody} body The request body.
   */
  void checkComment(HttpRequestBody body) {
    var result=_processBody(body);
    result['client'].checkComment(result['comment'])
      .then((isSpam) => _sendResponse(body.request.response, isSpam ? 'true' : 'false'))
      .catchError((error) => _sendResponse(body.request.response, 'invalid', error: error.message));
  }

  /**
   * Starts listening for HTTP requests.
   * @method start
   * @param {String|InternetAddress} address The address that the server is listening on.
   * @param {int} port The port that the server is listening on.
   * @return {Future} A `Future` that completes when the server has been started.
   * @async
   */
  Future start(address, int port) => HttpServer.bind(address, port)
    .then((server) {
      _server=server;
      _registerRoutes();
    })
    .catchError((error) {
      print('An error occurred while starting the server:');
      print(error);
      exit(1);
    });

  /**
   * Permanently stops this server from listening for new connections.
   * @method stop
   * @param {bool} [force=false] Value indicating whether active connections will be closed immediately.
   * @return {Future} A `Future` that completes when the server has been stopped.
   * @async
   */
  Future stop({ bool force: false }) {
    return _server==null ? new Future.value() : _server.close(force: force).then((_) { _server=null; });
  }

  /**
   * Submits a comment that was incorrectly marked as spam but should not have been.
   * @method submitHam
   * @param {http_server.HttpRequestBody} body The request body.
   */
  void submitHam(HttpRequestBody body) {
    var result=_processBody(body);
    result['client'].submitHam(result['comment'])
      .then((_) => _sendResponse(body.request.response, 'Thanks for making the web a better place.'))
      .catchError((error) => _sendResponse(body.request.response, '', error: error.message));
  }

  /**
   * Submits a comment that was not marked as spam but should have been.
   * @method submitSpam
   * @param {http_server.HttpRequestBody} body The request body.
   */
  void submitSpam(HttpRequestBody body) {
    var result=_processBody(body);
    result['client'].submitSpam(result['comment'])
      .then((_) => _sendResponse(body.request.response, 'Thanks for making the web a better place.'))
      .catchError((error) => _sendResponse(body.request.response, '', error: error.message));
  }

  /**
   * Checks an API key against the service database.
   * @method verifyKey
   * @param {http_server.HttpRequestBody} body The request body.
   */
  void verifyKey(HttpRequestBody body) {
    var result=_processBody(body);
    result['client'].verifyKey()
      .then((isValid) => _sendResponse(body.request.response, isValid ? 'valid' : 'invalid'))
      .catchError((error) => _sendResponse(body.request.response, 'invalid', error: error.message));
  }

  /**
   * Handles unmatched requests.
   * @method _defaultHandler
   * @param {HttpRequest} request The client request.
   * @private
   */
  void _defaultHandler(HttpRequest request) {
    // Handle CORS requests.
    if(request.method=='OPTIONS') {
      request.response
        ..statusCode=HttpStatus.NO_CONTENT
        ..close();

      return;
    }

    // Redirect the user whether specified.
    if(redirectUrl!=null) {
      request.response.redirect(redirectUrl.resolveUri(request.uri), status: HttpStatus.MOVED_PERMANENTLY);
      return;
    }

    // Send a 404 error for all other requests.
    request.response
      ..statusCode=HttpStatus.NOT_FOUND
      ..write('Not Found')
      ..close();
  }

  /**
   * Handles request errors.
   * @method _errorHandler
   * @param {dynamic} error The error that occurred.
   * @private
   */
  void _errorHandler(error) {
    var now=new DateTime.now();
    print('[$now] ERROR - $error');
  }

  /**
   * Processes the specified request body.
   * @method _processBody
   * @param {http_server.HttpRequestBody} body The request body.
   * @return {Map&lt;String, dynamic&gt;} The parsed `akismet.io.Client` and `akismet.core.Comment` in a map.
   * @private
   */
  Map<String, dynamic> _processBody(HttpRequestBody body) {
    var headers=body.request.headers;
    var query=body.body as Map;

    // Parse client arguments.
    var blog=Uri.parse('/');
    if(query.containsKey('blog')) {
      try { blog=Uri.parse(query['blog']); }
      catch(e) {}
    }

    var apiKey=(query.containsKey('key') ? query['key'] : headers.host.split('.').first);
    var userAgent=headers[core.HttpHeaders.USER_AGENT];

    var client=new Client(apiKey, blog)
      ..useSecureRequests=true
      ..userAgent=(userAgent.isEmpty ? headers[HttpHeaders.USER_AGENT] : userAgent).first;

    // Parse comment values.
    var comment=new core.Comment.fromJson(query);
    if(comment.author==null) comment.author=new core.Author();
    if(comment.author.ipAddress==null) comment.author.ipAddress=body.request.connectionInfo.remoteAddress.address;

    return { 'client': client, 'comment': comment };
  }

  /**
   * Logs the specified request and adds [CORS](http://www.w3.org/TR/cors) headers to the response.
   * @method _processRequest
   * @param {HttpRequest} request The client request.
   * @return {Future&lt;bool&gt;} A `Future` that always completes with the `true` value.
   * @async
   * @private
   */
  Future<bool> _processRequest(HttpRequest request) {
    // Log the request.
    var address=request.connectionInfo.remoteAddress.address;
    var now=new DateTime.now();
    var userAgent=request.headers[HttpHeaders.USER_AGENT].first;
    print('[$now] $address - "${request.method} ${request.uri} HTTP/${request.protocolVersion}" "$userAgent"');

    // Add CORS response headers.
    request.response
      ..headers.set('Access-Control-Allow-Headers', '${core.HttpHeaders.REQUESTED_WITH}, ${core.HttpHeaders.USER_AGENT}')
      ..headers.set('Access-Control-Allow-Methods', 'GET, OPTIONS, POST')
      ..headers.set('Access-Control-Allow-Origin', '*')
      ..headers.set('Access-Control-Expose-Headers', core.HttpHeaders.AKISMET_DEBUG_HELP);

    return new Future.value(true);
  }

  /**
   * Registers the route table.
   * @method _registerRoutes
   * @private
   */
  void _registerRoutes() {
    var bodyHandler=new HttpBodyHandler();
    new Router(_server)
      ..filter(new RegExp(r'^.*$'), _processRequest)
      ..serve(core.EndPoints.checkComment, method: 'POST')
        .transform(bodyHandler)
        .listen(checkComment, onError: _errorHandler)
      ..serve(core.EndPoints.submitHam, method: 'POST')
        .transform(bodyHandler)
        .listen(submitHam, onError: _errorHandler)
      ..serve(core.EndPoints.submitSpam, method: 'POST')
        .transform(bodyHandler)
        .listen(submitSpam, onError: _errorHandler)
      ..serve(core.EndPoints.verifyKey, method: 'POST')
        .transform(bodyHandler)
        .listen(verifyKey, onError: _errorHandler)
      ..defaultStream
        .listen(_defaultHandler, onError: _errorHandler);
  }

  /**
   * Writes the specified output to the specified server response.
   * @method _sendResponse
   * @param {HttpResponse} response The server response.
   * @param {String} output The output to be written.
   * @param {String} [error] An error message to be added to the response headers as `akismet.core.HttpHeaders.AKISMET_DEBUG_HELP`.
   * @private
   */
  void _sendResponse(HttpResponse response, String output, { String error }) {
    response.headers.contentType=new ContentType('text', 'plain', charset: UTF8.name);
    if(error!=null) response.headers.set(core.HttpHeaders.AKISMET_DEBUG_HELP, error);
    response.write(output);
    response.close();
  }
}
